"""Re-exported from oasyce_core. Do not add logic here."""
from oasyce_core.services.verification.api import *  # noqa: F401,F403
from oasyce_core.services.verification.api import (
    VerifyRequest, VerifyResponse, VerificationReport, VerificationStatus,
    _generate_asset_id, _build_message,
)
